--[[
A UnitGroupControl is a single button that is associated with a unit frame.
When the appropriate button combination is clicked on the unit frame, the unit
frame sends a Click message to the associated button.  That button then
transitions state, which signals the associated UnitGroupHeader, changing the
state of the header.  This in turn propogates to all ActionButton and
ActionButtonNone instances associated with that header, causing those buttons
to show.

There is one UnitGroupControl associated with each unit frame that registers.
With WoW 2.01, each registered frame gets a distinct UnitGroupControl instance.
With WoW 2.02, a registered frame gets the UnitGroupControl instance associated
with the unit the frame is associated with.  If the unit is not associated with
a frame, it reverts to the WoW 2.0.1 behavior.

Each UnitGroupControl references a single UnitGroupHeader.
--]]
uab.classes.UnitGroupControl = uab.mixins.aceOO.Class()

function uab.classes.UnitGroupControl.prototype:init(header, state, unit)
    uab.classes.UnitGroupControl.super.prototype.init(self)

	self.header = header
	self.state = state
	self.unit = unit
	self.enabled = true

	--[[
	Create the control frame
	--]]
    self.frame = CreateFrame("Button", self.header.frame:GetName().."Control"..self.state, self.header.frame, "SecureActionButtonTemplate")
    self.frame:SetAttribute("state", "0")
    self.frame.uabcontrol = self

    self:StatesInUseChanged()
    
    self.frame:SetScript("OnClick", function(self, button, down) self.uabcontrol:Clicked() end)
    self.frame:SetAttribute("useparent-unit", "true")
    self.frame:SetAttribute("useparent-unitsuffix", "true")

    self.header.frame:SetAttribute("addchild", self.frame)
end

function uab.classes.UnitGroupControl.prototype:StatesInUseChanged()
    if uab.db.profile.fastchangeunit then
        local maxinuse = self.header.controlsInUse
        local newstateval = "0:" .. self.state..";"..self.state..":"..self.state
        
        for idx = 1, maxinuse, 1 do
            if idx ~= self.state then
                newstateval = newstateval .. ";"..idx..":"..self.state
            end
        end
        
        self.frame:SetAttribute("newstate", newstateval)
    else
        self.frame:SetAttribute("newstate", "0:" .. self.state..";"..self.state..":"..self.state..";*:0")
    end
end

function uab.classes.UnitGroupControl.prototype:Clicked()
    local unit = SecureButton_GetModifiedUnit(self.unitframe)
    
    if unit ~= self.unitframeunit then
        if not self.unitframeunit then
            uab.pdebug("uab.classes.UnitGroupControl:Clicked() : unit changed from 'unknown' to '"..unit.."'")
        else
            uab.pdebug("uab.classes.UnitGroupControl:Clicked() : unit changed from '"..self.unitframeunit.."' to '"..unit.."'")
        end
        
        self.unitframeunit = unit
    end

--[[
    if unit then
        uab.print("uab.classes.UnitGroupControl:Clicked() : unit - "..unit..", state - "..self.state..", parentstate - "..self.frame:GetParent():GetAttribute("state"))
    else
        uab.print("uab.classes.UnitGroupControl:Clicked() : unit - unknown, state - "..self.state..", parentstate - "..self.frame:GetParent():GetAttribute("state"))
    end
--]]
	
    self.header:ShowActions()
end

function uab.classes.UnitGroupControl.prototype:Enable()
	if self.unitframe and not self.enabled then
		self:SetActivator()
		self.enabled = true
	end
end

function uab.classes.UnitGroupControl.prototype:Disable()
	if self.unitframe and self.enabled then
		self:ClearActivator()
		self.enabled = nil
	end
end

function uab.classes.UnitGroupControl.prototype:SwitchToDirectAction(action)
	local enabled = self.enabled
    
    self:Disable()

    self.directaction = action

    if enabled then
        self:Enable()
    end
end

function uab.classes.UnitGroupControl.prototype:SwitchToClickAction()
	local enabled = self.enabled
    
    self:Disable()

    self.directaction = nil

    if enabled then
        self:Enable()
    end
end

function uab.classes.UnitGroupControl.prototype:AssignFrame(frame)
    self:UnassignFrame()
    
    self.unitframe = frame
    self.unitframeunit = SecureButton_GetModifiedUnit(frame)
    
    if self.enabled then
		self:SetActivator()
	end
end

function uab.classes.UnitGroupControl.prototype:UnassignFrame()
	if self.unitframe then
    	if self.enabled then
			self:ClearActivator()
		end
		self.unitframe = nil
        self.unitframeunit = nil
	end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.UnitGroupControl.prototype:ClearActivator()
    for name, val in pairs(self.savedattrs) do
        self.unitframe:SetAttribute(name, val.oldval)
    end
    
    self.savedattrs = {}

--[[    
    self.unitframe:SetAttribute(self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode, self.savedattrs.type)
    self.unitframe:SetAttribute(self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode, self.savedattrs.clickbutton)
    self.savedattrs = {}
--]]
    
    if not self.directaction then
        self.header:ClearStateParent(self.state)
    end
end

function uab.classes.UnitGroupControl.prototype:SetActivator()
    self.savedattrs = {}
    
    if self.directaction then
        --[[
        These actions contain no modifiers.  We need to add our prefix and postfix to them before setting the values
        --]]        
        local attrs = self.directaction:GetData():GetAttributes()
        
    	for name, val in pairs(attrs) do
    		self.savedattrs[self.header.group.activator.prefix..name..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix..name..self.header.group.activator.buttoncode) }
    		self.unitframe:SetAttribute(self.header.group.activator.prefix..name..self.header.group.activator.buttoncode, val)
    	end
    else
        self.header:SetStateParent(self.state, self.unitframe)
    
        if not self.savedattrs then
            self.savedattrs = {}
        end
    
        self.savedattrs[self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode) }
        self.savedattrs[self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode) }

        self.unitframe:SetAttribute(self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode, "click")
        self.unitframe:SetAttribute(self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode, self.frame)
    end
end

